#!/usr/bin/env python3

# The code in this file is released under the Creative Commons CC0 license,
# i.e., for unrestricted use and without any warranty,
# see the file license_CC0.txt or https://creativecommons.org/publicdomain/zero/1.0/

import numpy as np
import matplotlib.pyplot as plt


def showPlotOfHistograms(uncertain_distance_values, uncertain_time_values, uncertain_velocity_values):
    # plot histograms of the values to check visually that the velocity values still follow
    # approximately a normal distribution despite the division
    _, axarr = plt.subplots(nrows=3, figsize=(8, 12))
    axarr[0].set_title("Distances")
    axarr[0].hist(uncertain_distance_values, bins=100)
    axarr[1].set_title("Times")
    axarr[1].hist(uncertain_time_values, bins=100)
    axarr[2].set_title("Velocities")
    axarr[2].hist(uncertain_velocity_values, bins=100)
    plt.show()


if __name__ == '__main__':
    np.random.seed(42)  # to get a reproducible sequence of pseudo-random numbers
    num_samples = 100000

    distance_value = 1000  # meters
    distance_uncertainty = 0.1 * distance_value  # 10 % uncertainty
    time_value = 100  # seconds
    time_uncertainty = 0.01 * time_value  # 1 % uncertainty

    # 1. Generate synthetic input samples within the range of uncertainty
    # (here, according to error model assuming normal distribution with given mean and standard deviation)
    uncertain_distance_values = [np.random.normal(distance_value, distance_uncertainty) for _ in range(num_samples)]
    uncertain_time_values = [np.random.normal(time_value, time_uncertainty) for _ in range(num_samples)]

    # 2. Perform entire computation for data evaluation
    # (here, this is just division)
    uncertain_velocity_values = np.divide(uncertain_distance_values, uncertain_time_values)  # entry-wise divide

    # 3. Visually inspect and plausibility-check input and output
    showPlotOfHistograms(uncertain_distance_values, uncertain_time_values, uncertain_velocity_values)

    # 4. Compute resulting uncertainty
    resulting_uncertainty = np.std(uncertain_velocity_values) / np.mean(uncertain_velocity_values) # coefficient of variation
    print(resulting_uncertainty)
    print("Resulting uncertainty is %5.2f%%" % (100 * resulting_uncertainty))
